#ifdef WIN32
#include <windows.h>
#endif

#include "GL/glew.h"
#include "glutil.h"

#include <fstream>

#include <sstream>
#include <iostream>


bool checkGLError(char *file, int line)
{
  bool wasError = false;

  for (GLenum glErr = glGetError(); glErr != GL_NO_ERROR; glErr = glGetError())
  {
    wasError = true;
    const GLubyte* sError = gluErrorString(glErr);
    
    if (!sError)
    {
      sError = reinterpret_cast<const GLubyte *>(" (no message available)");
    }

    std::cerr << "GL Error #" << glErr << "(" << sError << ") " << " in File " << file << " at line: " << line << std::endl;
    std::stringstream ss;
    ss  << file << "(" << line << "): GL Error #" << glErr << "(" << sError << ") " << std::endl;

    // outputs error message to debug console, if a debugger is attached.
    OutputDebugStringA(ss.str().c_str());
  }
  return wasError;
}

static bool ExtensionAvailable( const char *extString ) {
		char *extList = (char*) glGetString(GL_EXTENSIONS);		// Get the string containing a space-delimited list of all the supported extensions for the current hardware
		size_t extLen = strlen(extString);						// Get the size of our requested extension string for fast comparisons
		size_t nextExtLen = 0;									// Used to store the length of each extension in the list as we process it.

		if(!extString || !extList) { return false; }			// If we don't have either a extension list or requested extension, return false 
		
		while(*extList) {										// Loop through each extention in the list till we find one that matches the requested extension
			nextExtLen = strcspn( extList, " " );				// Get the number of charecters up to the next space (which serves as a delimiter in the extension list)
			if(nextExtLen == extLen && strncmp( extList, extString, extLen ) == 0) {	// We first mach the string length, then do a char-by-char comparison
				return true;									// If the extension matches the requested one, return true
			}
			extList += nextExtLen + 1;							// If this extension did not match, move forward to the next extension string
		}

		return false;											// If we reach here we haven't found the extension in our list, so return false
	}

void BindExtensions( void ) {
	// Check to see if Vertex Buffer Objects are supported by the current hardware/drivers, If not, exit the application.
	if( !ExtensionAvailable("GL_ARB_vertex_buffer_object") ) {		
		throw std::exception("This tutorial requires Vertex Buffer Objects, which are not supported by the current driver");
	}

/*	// Retrieve the function pointers for each of the extension functions we will be using
	glGenBuffersARB		= (PFNGLGENBUFFERSARBPROC) wglGetProcAddress("glGenBuffersARB");			
	glBindBufferARB		= (PFNGLBINDBUFFERARBPROC) wglGetProcAddress("glBindBufferARB");
	glBufferDataARB		= (PFNGLBUFFERDATAARBPROC) wglGetProcAddress("glBufferDataARB");
	glDeleteBuffersARB	= (PFNGLDELETEBUFFERSARBPROC) wglGetProcAddress("glDeleteBuffersARB");
	
	// Check to see if the GLSL shading language is supported by the current hardware/drivers, If not, exit the application.
	if( !ExtensionAvailable("GL_ARB_shading_language_100") ) {
		throw std::exception("This tutorial requires GLSL Shaders, which are not supported by the current driver");
	}
	
	glCreateProgramObjectARB  = (PFNGLCREATEPROGRAMOBJECTARBPROC)wglGetProcAddress("glCreateProgramObjectARB");
	glDeleteObjectARB         = (PFNGLDELETEOBJECTARBPROC)wglGetProcAddress("glDeleteObjectARB");
	glUseProgramObjectARB     = (PFNGLUSEPROGRAMOBJECTARBPROC)wglGetProcAddress("glUseProgramObjectARB");
	glCreateShaderObjectARB   = (PFNGLCREATESHADEROBJECTARBPROC)wglGetProcAddress("glCreateShaderObjectARB");
	glShaderSourceARB         = (PFNGLSHADERSOURCEARBPROC)wglGetProcAddress("glShaderSourceARB");
	glCompileShaderARB        = (PFNGLCOMPILESHADERARBPROC)wglGetProcAddress("glCompileShaderARB");
	glGetObjectParameterivARB = (PFNGLGETOBJECTPARAMETERIVARBPROC)wglGetProcAddress("glGetObjectParameterivARB");
	glAttachObjectARB         = (PFNGLATTACHOBJECTARBPROC)wglGetProcAddress("glAttachObjectARB");
	glGetInfoLogARB           = (PFNGLGETINFOLOGARBPROC)wglGetProcAddress("glGetInfoLogARB");
	glLinkProgramARB          = (PFNGLLINKPROGRAMARBPROC)wglGetProcAddress("glLinkProgramARB");
	*/
}

// Helper function used to get log info (such as errors) about a shader object or shader program
std::string GetShaderInfoLog(GLhandleARB& obj) {
	int logLength = 0;
	int charsWritten  = 0;
	char *tmpLog;
	std::string log;

	glGetObjectParameterivARB(obj, GL_OBJECT_INFO_LOG_LENGTH_ARB, &logLength);

	if (logLength > 0) {
		tmpLog = new char[logLength];
		glGetInfoLogARB(obj, logLength, &charsWritten, tmpLog);
		log = tmpLog;
		delete[] tmpLog;
	}

	return log;
}

// Helper function used to create a shader object from text in a file
char *textFileRead(char *fn) {
	FILE *fp;
	char *content = NULL;
	int count=0;

	if (fn != NULL) {
		fp = fopen(fn,"rt");
		if (fp != NULL) {
      fseek(fp, 0, SEEK_END);
      count = ftell(fp);
      rewind(fp);
			if (count > 0) {
				content = (char *)malloc(sizeof(char) * (count+1));
				count = fread(content,sizeof(char),count,fp);
				content[count] = '\0';
			}
			fclose(fp);
		}
	}
	return content;
}

