
#include "gl/gl.h"
#include "gl/glext.h"

#include <fstream>

char *textFileRead(char *fn);
std::string GetShaderInfoLog(GLhandleARB& obj);


#ifndef LAB_GL_UTILS_H
#define LAB_GL_UTILS_H

/**
 * This file contains utility functions to be used for the labs in computer graphics at chalmers,
 * they are not covered by any particular license...
 */

/**
 * This macro checks for gl errors using glGetError, it is useful to sprinkle it around the code base, especially
 * when unsure about the correct usage, for example after each call to open gl.
 * When the debugger is atached it will cause a break on the offending line, and also print out the file 
 * and line in a MSVC compatible format on the debug output and console.
 *
 * Note: the macro _cannot_ be used between glBegin/glEnd brackets, as stated by the openGL standard.
 * Note2: be aware that the macro will report any previous errors, since the last call to glGetError.
 *
 * example usage: glClear(GL_COLOR_BUFFER_BIT); CHECK_GL_ERROR(); // this will check for errors in this (and any previous statements)
 */
#define CHECK_GL_ERROR() { checkGLError(__FILE__, __LINE__) && (__debugbreak(), 1); }

/**
 * Internal function used by macro CHECK_GL_ERROR, use that instead.
 */
bool checkGLError(char *file, int line);


#endif // LAB_GL_UTILS_H
